@@ -1,19 +1,84 @@
 #' Inset tables
 #'
-#' \code{geom_table} adds a textual table directly to the ggplot using syntax
-#' similar to that of \code{\link[ggplot2]{geom_label}} while
-#' \code{geom_table_npc} is similar to \code{geom_label_npc} in that \code{x}
-#' and \code{y} coordinates are given in npc units. In most respects they
-#' behave as any other ggplot geometry: a layer con contain multiple tables
-#' and faceting works as usual.
+#' \code{geom_table} and \code{geom_table_npc} add data frames as table insets
+#' to the base ggplot, using syntax similar to that of
+#' \code{\link[ggplot2]{geom_text}} and \code{\link{geom_text_s}}. In most
+#' respects they behave as any other ggplot geometry: a layer con contain
+#' multiple tables and faceting works as usual.
 #'
-#' @section Alignment: You can modify table alignment with the \code{vjust} and
-#'   \code{hjust} aesthetics. These can either be a number between 0
-#'   (right/bottom) and 1 (top/left) or a character ("left", "middle", "right",
-#'   "bottom", "center", "top").
+#' @details You can modify the alignment of inset tables with the \code{vjust}
+#'   and \code{hjust} aesthetics. These can either be a number between 0
+#'   (right/bottom) and 1 (top/left) or a character
+#'   ("left", "middle", "right", "bottom", "center", "top").
 #'
-#' @section Inset size: You can modify inset table size with the \code{size}
-#'   aesthetics, which determines the size of text within the table.
+#'   You can modify the size of inset tabls with the \code{vp.width} and
+#'   \code{vp.height} aesthetics. These can take a number between 0 (smallest
+#'   possible inset) and 1 (whole plotting area width or height). The default
+#'   value for for both of these aesthetics is 1/5. Thus, in contrast to
+#'   \code{\link[ggplot2]{geom_text}} and \code{\link{geom_text_s}} the size of
+#'   the insets remains the same relative to the size of the plotting area
+#'   irrespective of how the plot is rendered. The aspect ratio of insets is
+#'   preserved and size is adjusted until the whole inset fits within the
+#'   viewport.
+#'
+#'   You can modify inset table alignment with the `vjust` and `hjust`
+#'   aesthetics. These can either be a number between 0 (right/bottom) and 1
+#'   (top/left) or a character (`"left"`, `"middle"`, `"right"`, `"bottom"`,
+#'   `"center"`, `"top"`). There several two special alignments: `"inward"` and
+#'   `"outward"`. Inward always aligns text towards the center of the plotting
+#'   area, and outward aligns it away from the center of the plotting area. It
+#'   tagged with `_mean` or `_median` the mean or median of the data in the
+#'   panel along the corresponding axis is used as center.
+#'
+#'   By default this geom uses \code{\link{position_nudge_center}} which is
+#'   backwards compatible with \code{\link[ggplot2]{position_nudge}} but
+#'   provides additional control on the direction of the nudging. In contrast to
+#'   \code{\link[ggplot2]{position_nudge}}, \code{\link{position_nudge_center}}
+#'   and all other position functions defined in packages 'ggpp' and 'ggrepel'
+#'   keep the original coordinates thus allowing the plotting of connecting
+#'   segments and arrows.
+#'
+#'   This geom works only with tibbles as \code{data}, as its expects a list of
+#'   data frames (or tibbles) to be mapped to the \code{label} aesthetic.
+#'   A table is built with function \code{gridExtra::gtable()} for each
+#'   data frame in the list, and formatted according to a table theme or
+#'   \code{ttheme}. The character strings in the data frame can be parsed into
+#'   R expressions so the inset tables can include maths.
+#'
+#'   If the argument passed to \code{table.theme} is a constructor function
+#'   (passing its name without parenthesis), the values mapped to \code{size},
+#'   \code{colour}, \code{fill}, \code{alpha}, and \code{family} aesthetics will
+#'   the passed to this theme constructor for each individual table. In
+#'   contrast, if a ready constructed \code{ttheme} stored as a list object is
+#'   passed as argument (e.g., by calling the constructor, using constructor
+#'   name followed by parenthesis), it will be used as is, i.e., mappings to
+#'   aesthetics such as \code{colour} are ignored if present. By default the
+#'   constructor \code{ttheme_gtdefault} is used and \code{colour} and
+#'   \code{fill}, are mapped to \code{NA}. Mapping these aesthetics to \code{NA}
+#'   triggers the use of the default base_colour of the \code{ttheme}.
+#'   As the table is built with function \code{gridExtra::gtable()}, for
+#'   formatting details, please, consult \code{\link[gridExtra]{tableGrob}}.
+#'
+#'   The \code{x} and \code{y} aesthetics determine the position of the whole
+#'   inset table, similarly to that of a text label, justification is interpreted
+#'   as indicating the position of the inset table with respect to its x and y
+#'   coordinates in the data, and \code{angle} is used to rotate the inset table as a
+#'   whole.
+#'
+#'   In the case of \code{geom_table_npc()}, \code{npcx} and \code{npcy}
+#'   aesthetics determine the position of the inset table. As for text labels,
+#'   justification is interpreted as indicating the position of the inset plot
+#'   with respect to its \code{npcx} and \code{npcy} coordinates, and
+#'   \code{angle} is used to rotate the plot as a whole.
+#'
+#'   \code{\link[ggplot2]{annotate}} cannot be used with \code{geom = "table"}.
+#'   Use \code{\link{annotate}} (automatic override unless 'ggpp' is not
+#'   attached) as redefined in 'ggpp' when adding inset plots as annotations
+#'   (automatic unless 'ggpp' is not attached).
+#'
+#' @seealso Formatting of tables \code{stat_fmt_table},
+#'   \code{\link{ttheme_gtdefault}}, \code{\link{ttheme_set}},
+#'   \code{\link[gridExtra]{tableGrob}}.
 #'
 #' @param mapping The aesthetic mapping, usually constructed with
 #'   \code{\link[ggplot2]{aes}} or \code{\link[ggplot2]{aes_}}. Only needs
@@ -44,64 +109,18 @@
 #'   than combining with them. This is most useful for helper functions that
 #'   define both data and aesthetics and shouldn't inherit behaviour from the
 #'   default plot specification, e.g. \code{\link[ggplot2]{borders}}.
+#' @param nudge_x,nudge_y Horizontal and vertical adjustments to nudge the
+#'   starting position of each text label. The units for \code{nudge_x} and
+#'   \code{nudge_y} are the same as for the data units on the x-axis and y-axis.
+#' @param add.segments logical Display connecting segments or arrows between
+#'   original positions and displaced ones if both are available.
+#' @param arrow specification for arrow heads, as created by
+#'   \code{\link[grid]{arrow}}
 #'
-#' @details These geoms work only with tibbles as \code{data}, as they expects a
-#'   list of data frames or tibbles ("tb" objects) to be mapped to the
-#'   \code{label} aesthetic. Aesthetics mappings in the inset plot are
-#'   independent of those in the base plot.
-#'
-#'   In the case of \code{geom_table()}, \code{x} and \code{y} aesthetics
-#'   determine the position of the whole inset table, similarly to that of a
-#'   text label, justification is interpreted as indicating the position of the
-#'   table with respect to the $x$ and $y$ coordinates in the data, and
-#'   \code{angle} is used to rotate the table as a whole.
-#'
-#'   In the case of \code{geom_table_npc()}, \code{npcx} and \code{npcy}
-#'   aesthetics determine the position of the whole inset table, similarly to
-#'   that of a text label, justification is interpreted as indicating the
-#'   position of the table with respect to the $x$ and $y$ coordinates in "npc"
-#'   units, and \code{angle} is used to rotate the table as a whole.
-#'
-#'   The "width" and "height" of an inset as for a text element are
-#'   0, so stacking and dodging inset tables will not work by default, and axis
-#'   limits are not automatically expanded to include all inset tables.
-#'   Obviously, insets do have height and width, but they are physical units,
-#'   not data units. The amount of space they occupy on the main plot is not
-#'   constant in data units of the base plot: when you modify scale limits,
-#'   inset plots stay the same size relative to the physical size of the base
-#'   plot.
-#'
-#'   If the argument passed to \code{table.theme} is a constructor function
-#'   (passing its name without parenthesis), the values mapped to \code{size},
-#'   \code{colour}, \code{fill}, \code{alpha}, and \code{family} aesthetics will
-#'   the passed to this theme constructor for each individual table. In
-#'   contrast, if a ready constructed ttheme as a list object is passed as
-#'   argument (e.g., by calling the constructor, using contructor name followed
-#'   by parenthesis), it will be used as is, i.e., mappings to aesthetics such
-#'   as \code{colour} are ignored if present.
-#'
-#'   By default the constructor \code{ttheme_gtdefault} is used and
-#'   \code{colour} and \code{fill}, are mapped to \code{NA}. Mapping these
-#'   aesthetics to \code{NA} triggers the use of the default base_colour of the
-#'   \code{ttheme}.
-#'
-#'   As the table is built with function \code{gridExtra::gtable()}, for
-#'   formatting details, please, consult \code{\link[gridExtra]{tableGrob}}.
-#'
-#' @note As all geometries, \code{geom_table()} and \code{geom_table_npc()} add
-#'   a layer to a plot, and behave as expected in the grammar of graphics. In general
-#'   ggplot themes do not affect how layers are rendered, and this is also the
-#'   case for \code{geom_table()}. As described abovem, the formatting of the
-#'   inset table is done according to the the argument passed to parameter
-#'   \code{table.theme}.
-#'
-#'   Complex tables with annotations or different colouring of rows or cells can
-#'   be constructed with functions in package 'gridExtra' or in any other way
-#'   as long as they can be saved as grid graphical objects and then added to a
-#'   ggplot as a new layer with \code{\link{geom_grob}}.
-#'
-#'   In 'ggpmisc' (>= 0.3.6) \strong{\code{annotate()} can be used with
-#'   \code{geom = "table"}}.
+#' @note Complex tables with annotations or different colouring of rows or cells
+#'   can be constructed with functions in package 'gridExtra' or in any other
+#'   way as long as they can be saved as grid graphical objects and then added
+#'   to a ggplot as a new layer with \code{\link{geom_grob}}.
 #'
 #' @references This geometry is inspired on answers to two questions in
 #'   Stackoverflow. In contrast to these earlier examples, the current geom
@@ -110,9 +129,6 @@
 #'   \url{https://stackoverflow.com/questions/12318120/adding-table-within-the-plotting-region-of-a-ggplot-in-r}
 #'   \url{https://stackoverflow.com/questions/25554548/adding-sub-tables-on-each-panel-of-a-facet-ggplot-in-r?}
 #'
-#' @seealso function \code{\link[gridExtra]{tableGrob}} as it is used to
-#'   construct the table.
-#'
 #' @family geometries adding layers with insets
 #'
 #' @return A plot layer instance.
@@ -156,7 +172,11 @@
 #'              table.theme = ttheme_gtminimal) +
 #'   theme_classic()
 #'
-#' df2 <- tibble(x = 5.45, y = c(34, 29, 24), cyl = c(4, 6, 8),
+#' df2 <- tibble(x = 5.45,
+#'               y = c(34, 29, 24),
+#'               x1 = c(2.29, 3.12, 4.00),
+#'               y1 = c(26.6, 19.7, 15.1),
+#'               cyl = c(4, 6, 8),
 #'               tb = list(tb[1, 1:3], tb[2, 1:3], tb[3, 1:3]))
 #'
 #' # mapped aesthetics
@@ -166,6 +186,17 @@
 #'              inherit.aes = TRUE,
 #'              mapping = aes(x = x, y = y, label = tb))
 #'
+#' # nudging and segments
+#' ggplot(data = mtcars, mapping = aes(wt, mpg, color = factor(cyl))) +
+#'   geom_point(show.legend = FALSE) +
+#'   geom_table(data = df2,
+#'              inherit.aes = TRUE,
+#'              nudge_x = 0.7, nudge_y = 3,
+#'              vjust = 0.5, hjust = 0.5,
+#'              arrow = arrow(length = unit(0.5, "lines")),
+#'              mapping = aes(x = x1, y = y1, label = tb)) +
+#'   theme_classic()
+#'
 #' # Using native plot coordinates instead of data coordinates
 #' dfnpc <- tibble(x = 0.95, y = 0.95, tb = list(tb))
 #'
@@ -176,6 +207,10 @@
 geom_table <- function(mapping = NULL, data = NULL,
                        stat = "identity", position = "identity",
                        ...,
+                       nudge_x = 0,
+                       nudge_y = 0,
+                       add.segments = TRUE,
+                       arrow = NULL,
                        table.theme = NULL,
                        table.rownames = FALSE,
                        table.colnames = TRUE,
@@ -184,6 +219,18 @@ geom_table <- function(mapping = NULL, data = NULL,
                        na.rm = FALSE,
                        show.legend = FALSE,
                        inherit.aes = FALSE) {
+
+  if (!missing(nudge_x) || !missing(nudge_y)) {
+    if (!missing(position)) {
+      rlang::abort("You must specify either `position` or `nudge_x`/`nudge_y`.")
+    }
+    # We do not keep the original positions if they will not be used
+    position <-
+      position_nudge_center(nudge_x, nudge_y,
+                            kept.origin = ifelse(add.segments,
+                                                 "original", "none"))
+  }
+
   if (is.character(table.hjust)) {
     table.hjust <- switch(table.hjust,
                           left = 0,
@@ -192,7 +239,8 @@ geom_table <- function(mapping = NULL, data = NULL,
                           right = 1,
                           0.5)
   }
-  layer(
+
+  ggplot2::layer(
     data = data,
     mapping = mapping,
     stat = stat,
@@ -201,6 +249,8 @@ geom_table <- function(mapping = NULL, data = NULL,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
     params = list(
+      add.segments = add.segments,
+      arrow = arrow,
       table.theme = table.theme,
       table.rownames = table.rownames,
       table.colnames = table.colnames,
@@ -223,6 +273,8 @@ gtb_draw_panel_fun <-
   function(data,
            panel_params,
            coord,
+           add.segments = TRUE,
+           arrow = NULL,
            table.theme = NULL,
            table.rownames = FALSE,
            table.colnames = TRUE,
@@ -240,8 +292,17 @@ gtb_draw_panel_fun <-
       return(grid::nullGrob())
     }
 
+    add.segments <- add.segments && all(c("x_orig", "y_orig") %in% colnames(data))
+
     # should be called only once!
     data <- coord$transform(data, panel_params)
+    if (add.segments) {
+      data_orig <- data.frame(x = data$x_orig, y = data$y_orig)
+      data_orig <- coord$transform(data_orig, panel_params)
+      data$x_orig <- data_orig$x
+      data$y_orig <- data_orig$y
+    }
+
     if (is.character(data$vjust)) {
       data$vjust <-
         compute_just2d(data = data,
@@ -265,29 +326,32 @@ gtb_draw_panel_fun <-
         getOption("ggpmisc.ttheme.default", default = ttheme_gtdefault)
     }
 
-    tb.grobs <- grid::gList()
+    # loop needed as gpar is not vectorized
+    all.grobs <- grid::gList()
 
-    for (row.idx in seq_len(nrow(data))) {
-      # if needed, construct the table theme
+    for (row.idx in 1:nrow(data)) {
+      row <- data[row.idx, , drop = FALSE]
+
+      # Build the table
       if (is.function(table.theme)) {
-      # tableGrob puts all the padding on the same side unless just = 0.5
-      # this makes it difficult to compute a suitable value for table.x
-      # without knowing the width of the column. The code here at least
-      # ensures that whatever its length the whole text is always displayed.
+        # tableGrob puts all the padding on the same side unless just = 0.5
+        # this makes it difficult to compute a suitable value for table.x
+        # without knowing the width of the column. The code here at least
+        # ensures that whatever its length the whole text is always displayed.
         table.x <- table.hjust
-        if (is.na(data$fill[[row.idx]])) {
+        if (is.na(row$fill)) {
           core.params <-
             list(fg_params = list(hjust = table.hjust, x = table.x))
         } else {
           core.params <-
             list(fg_params = list(hjust = table.hjust, x = table.x),
-                 bg_params = list(fill = data$fill[row.idx]))
+                 bg_params = list(fill = row$fill))
         }
-        if (is.na(data$colour[row.idx])) {
+        if (is.na(row$colour)) {
           # use theme's default base_colour
           this.table.theme <-
-            table.theme(base_size = data$size[row.idx] * .pt,
-                        base_family = data$family[[row.idx]],
+            table.theme(base_size = row$size * .pt,
+                        base_family = row$family,
                         parse = parse,
                         rowhead = list(fg_params = list(hjust = 1, x = 0.9)),
                         colhead = list(fg_params = list(hjust = table.hjust,
@@ -296,10 +360,9 @@ gtb_draw_panel_fun <-
         } else {
           this.table.theme <-
             # use colour from data$colour
-            table.theme(base_size = data$size[row.idx] * .pt,
-                        base_colour = ggplot2::alpha(data$colour[row.idx],
-                                                     data$alpha[row.idx]),
-                        base_family = data$family[[row.idx]],
+            table.theme(base_size = row$size * .pt,
+                        base_colour = ggplot2::alpha(row$colour, row$alpha),
+                        base_family = row$family,
                         parse = parse,
                         rowhead = list(fg_params = list(hjust = 1, x = 0.9)),
                         colhead = list(fg_params = list(hjust = table.hjust,
@@ -310,7 +373,7 @@ gtb_draw_panel_fun <-
         this.table.theme <- table.theme
       }
       table.tb <- data[["label"]][[row.idx]]
-      gtb <-
+      user.grob <-
         gridExtra::tableGrob(
           d = table.tb,
           theme = this.table.theme,
@@ -318,26 +381,37 @@ gtb_draw_panel_fun <-
           cols = if (table.colnames) colnames(table.tb) else NULL
         )
 
-      gtb$vp <-
-        grid::viewport(x = grid::unit(data$x[row.idx], "native"),
-                       y = grid::unit(data$y[row.idx], "native"),
-                       width = sum(gtb$widths),
-                       height = sum(gtb$heights),
-                       just = c(data$hjust[row.idx], data$vjust[row.idx]),
-                       angle = data$angle[row.idx],
-                       name = paste("geom_table.panel", data$PANEL[row.idx],
+      user.grob$vp <-
+        grid::viewport(x = grid::unit(row$x, "native"),
+                       y = grid::unit(row$y, "native"),
+                       width = sum(user.grob$widths),
+                       height = sum(user.grob$heights),
+                       just = c(row$hjust, row$vjust),
+                       angle = row$angle,
+                       name = paste("inset.table.vp", row$PANEL,
                                     "row", row.idx, sep = "."))
 
-      # give unique name to each table
-      gtb$name <- paste("table", row.idx, sep = ".")
+      # give unique name to each grob
+      user.grob$name <- paste("inset.table", row.idx, sep = ".")
 
-      tb.grobs[[row.idx]] <- gtb
+      if (add.segments) {
+        segment.grob <-
+          grid::segmentsGrob(x0 = row$x,
+                             y0 = row$y,
+                             x1 = row$x_orig,
+                             y1 = row$y_orig,
+                             arrow = arrow,
+                             gp = grid::gpar(col = ggplot2::alpha(row$segment.colour,
+                                                                  row$segment.alpha)),
+                             name = paste("inset.table.segment", row.idx, sep = "."))
+        all.grobs <- grid::gList(all.grobs, segment.grob, user.grob)
+      } else {
+        all.grobs <- grid::gList(all.grobs, user.grob)
+      }
     }
 
-    grid.name <- paste("geom_table.panel",
-                       data$PANEL[row.idx], sep = ".")
+    grid::grobTree(children = all.grobs, name = "geom.table.panel")
 
-    grid::gTree(children = tb.grobs, name = grid.name)
   }
 
 #' @rdname ggpp-ggproto
@@ -345,14 +419,24 @@ gtb_draw_panel_fun <-
 #' @usage NULL
 #' @export
 GeomTable <-
-  ggproto("GeomTable", Geom,
+  ggplot2::ggproto("GeomTable", ggplot2::Geom,
           required_aes = c("x", "y", "label"),
 
-          default_aes = aes(
-            colour = NA, fill = NA,
-            size = 3.2, angle = 0, hjust = "inward",
-            vjust = "inward", alpha = 1, family = "", fontface = 1,
-            lineheight = 1.2
+          default_aes = ggplot2::aes(
+            colour = NA,
+            fill = NA,
+            size = 3.2,
+            angle = 0,
+            hjust = "inward",
+            vjust = "inward",
+            alpha = 1,
+            family = "",
+            fontface = 1,
+            lineheight = 1.2,
+            segment.linetype = 1,
+            segment.colour = "grey33",
+            segment.size = 0.5,
+            segment.alpha = 1
           ),
 
           draw_panel = gtb_draw_panel_fun,
@@ -383,7 +467,7 @@ geom_table_npc <- function(mapping = NULL, data = NULL,
                           right = 1,
                           0.5)
   }
-  layer(
+  ggplot2::layer(
     data = data,
     mapping = mapping,
     stat = stat,
@@ -513,10 +597,7 @@ gtbnpc_draw_panel_fun <-
       tb.grobs[[row.idx]] <- gtb
     }
 
-    grid.name <- paste("geom_table.panel",
-                       data$PANEL[row.idx], sep = ".")
-
-    grid::gTree(children = tb.grobs, name = grid.name)
+    grid::grobTree(children = tb.grobs)
   }
 
 #' @rdname ggpp-ggproto
@@ -524,10 +605,10 @@ gtbnpc_draw_panel_fun <-
 #' @usage NULL
 #' @export
 GeomTableNpc <-
-  ggproto("GeomTableNpc", Geom,
+  ggplot2::ggproto("GeomTableNpc", ggplot2::Geom,
           required_aes = c("npcx", "npcy", "label"),
 
-          default_aes = aes(
+          default_aes = ggplot2::aes(
             colour = NA, fill = NA,
             size = 3.2, angle = 0, hjust = "inward",
             vjust = "inward", alpha = 1, family = "", fontface = 1,
